"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CacheMedia = void 0;
const crypto_1 = __importDefault(require("crypto"));
const path_1 = __importDefault(require("path"));
const axios_1 = __importDefault(require("axios"));
const cheerio_1 = __importDefault(require("cheerio"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const sharp_1 = __importDefault(require("sharp"));
const util_1 = require("./util");
class CacheMedia {
    constructor(dst) {
        this.dst = dst;
    }
    parse(html) {
        return __awaiter(this, void 0, void 0, function* () {
            const $ = cheerio_1.default.load(html);
            yield Promise.all(Array.from($('img:not([data-no-cache])')).map((el) => __awaiter(this, void 0, void 0, function* () {
                const $el = $(el);
                yield this.localizeImage($el);
            })));
            return $('body').html() || html;
        });
    }
    minimizeImage(data, $el) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield sharp_1.default(data)
                .resize(($el
                ? parseInt($el.attr('width') || '') ||
                    parseInt($el.attr('data-width') || '') ||
                    util_1.styleSizeToNumber($el.css('width'))
                : null) || 800, $el
                ? parseInt($el.attr('height') || '') ||
                    parseInt($el.attr('data-height') || '') ||
                    util_1.styleSizeToNumber($el.css('height'))
                : null, {
                withoutEnlargement: true,
                fit: 'outside',
            })
                // .toFormat('webp', { quality: 80 })
                .toBuffer();
        });
    }
    /**
     *
     * @param im If used externally, it means full URL. Internally, it uses Cheerio.
     */
    localizeImage(im) {
        return __awaiter(this, void 0, void 0, function* () {
            let src = '';
            let $el = null;
            if (typeof im === 'string') {
                src = im;
            }
            else {
                $el = im;
                src = im.attr('src') || '';
            }
            if (src && util_1.isUrl(src)) {
                try {
                    const { data } = yield axios_1.default.get(src, {
                        responseType: 'arraybuffer',
                    });
                    const newUrl = `${crypto_1.default
                        .createHash('sha256')
                        .update(data)
                        .digest('hex')}/${util_1.extractFilenameFromUrl(src, 'image.png', {
                        preferredExt: ['.jpg', '.gif', '.png', '.jpeg', '.webp'],
                    })}`;
                    if ($el) {
                        $el.attr('src', `/media/${newUrl}`);
                        $el.attr('data-original-src', src);
                    }
                    yield fs_extra_1.default.ensureFile(path_1.default.join(this.dst, newUrl));
                    yield fs_extra_1.default.writeFile(path_1.default.join(this.dst, newUrl), yield this.minimizeImage(data, $el));
                    return newUrl;
                }
                catch (_) { }
            }
            return null;
        });
    }
}
exports.CacheMedia = CacheMedia;
//# sourceMappingURL=cache.js.map