"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MakeHtml = void 0;
const highlightjs_vue_1 = require("@patarapolw/highlightjs-vue");
const markdown_it_imsize_1 = __importDefault(require("@patarapolw/markdown-it-imsize"));
const cheerio_1 = __importDefault(require("cheerio"));
const highlight_js_1 = __importDefault(require("highlight.js"));
const hyperpug_1 = __importDefault(require("hyperpug"));
const markdown_it_1 = __importDefault(require("markdown-it"));
const markdown_it_container_1 = __importDefault(require("markdown-it-container"));
const markdown_it_emoji_1 = __importDefault(require("markdown-it-emoji"));
const markdown_it_external_links_1 = __importDefault(require("markdown-it-external-links"));
const utils_1 = require("markdown-it/lib/common/utils");
const stylis_1 = __importDefault(require("stylis"));
const card_1 = require("./components/card");
class MakeHtml {
    constructor(id = Math.random().toString(36)) {
        this.id = id;
        this.html = '';
        highlightjs_vue_1.hljsRegisterVue(highlight_js_1.default);
        this.id = 'el-' + hashFnv32a(this.id);
        this.md = markdown_it_1.default({
            breaks: true,
            html: true,
            highlight: (str, lang) => {
                if (lang && highlight_js_1.default.getLanguage(lang)) {
                    try {
                        return ('<pre class="hljs"><code>' +
                            highlight_js_1.default.highlight(lang, str, true).value +
                            '</code></pre>');
                    }
                    catch (__) { }
                }
                return ('<pre class="hljs"><code>' +
                    this.md.utils.escapeHtml(str) +
                    '</code></pre>');
            },
        })
            .use((md) => {
            const { fence } = md.renderer.rules;
            md.renderer.rules.fence = (tokens, idx, options, env, slf) => {
                const token = tokens[idx];
                const info = token.info ? utils_1.unescapeAll(token.info).trim() : '';
                const content = token.content;
                if (info === 'pug parsed') {
                    return this._pugConvert(content);
                }
                return fence(tokens, idx, options, env, slf);
            };
            return md;
        })
            .use(markdown_it_emoji_1.default)
            .use(markdown_it_imsize_1.default)
            .use(markdown_it_external_links_1.default, {
            externalTarget: '_blank',
            externalRel: 'noopener nofollow noreferrer',
        })
            .use(markdown_it_container_1.default, 'spoiler', {
            validate: (params) => {
                return params.trim().match(/^spoiler(?:\s+(.*))?$/);
            },
            render: (tokens, idx) => {
                const m = tokens[idx].info.trim().match(/^spoiler(?:\s+(.*))?$/);
                if (tokens[idx].nesting === 1) {
                    // opening tag
                    return ('<details><summary>' +
                        this.md.utils.escapeHtml(m[1] || 'Spoiler') +
                        '</summary>\n');
                }
                else {
                    // closing tag
                    return '</details>\n';
                }
            },
        });
        this.hp = new hyperpug_1.default({
            markdown: (s) => this._mdConvert(s),
        });
    }
    render(s) {
        try {
            if (s.startsWith('---\n')) {
                s = s.substr(4).split(/---\n(.*)$/s)[1] || '';
            }
            this.html = this._mdConvert(s);
        }
        catch (e) { }
        const $ = cheerio_1.default.load(this.html);
        $('style').each((_, el) => {
            const $el = $(el);
            const innerHTML = $el.html();
            if (innerHTML) {
                $el.html(stylis_1.default(`.${this.id}`, innerHTML));
            }
            else {
                $el.remove();
            }
        });
        $('iframe').each((_, el) => {
            const $el = $(el);
            const w = $el.attr('width');
            const h = $el.attr('height');
            $el.css({
                width: w ? `${w}px` : undefined,
                height: h ? `${h}px` : undefined,
            });
        });
        $('img, iframe').each((_, el) => {
            $(el).attr('loading', 'lazy');
        });
        $('a[data-make-html="card"]').each((_, el) => {
            card_1.compileCardComponent($(el));
        });
        return `<div class="${this.id}">${$('body').html() || ''}</div>`;
    }
    _pugConvert(s) {
        return this.hp.parse(s);
    }
    _mdConvert(s) {
        return this.md.render(s);
    }
}
exports.MakeHtml = MakeHtml;
/**
 * Calculate a 32 bit FNV-1a hash
 * Found here: https://gist.github.com/vaiorabbit/5657561
 * Ref.: http://isthe.com/chongo/tech/comp/fnv/
 *
 * @param {string} str the input value
 * @param {integer} [seed] optionally pass the hash of the previous chunk
 * @returns {string}
 */
function hashFnv32a(str, seed) {
    /* jshint bitwise:false */
    var i;
    var l;
    var hval = seed === undefined ? 0x811c9dc5 : seed;
    for (i = 0, l = str.length; i < l; i++) {
        hval ^= str.charCodeAt(i);
        hval += (hval << 1) + (hval << 4) + (hval << 7) + (hval << 8) + (hval << 24);
    }
    return (hval >>> 0).toString(36);
}
//# sourceMappingURL=frontend.js.map