"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractFilenameFromUrl = exports.styleSizeToNumber = exports.isUrl = void 0;
const sanitize_filename_1 = __importDefault(require("sanitize-filename"));
function isUrl(s) {
    if (/^https?:\/\/[^ ]+$/.test(s)) {
        try {
            // eslint-disable-next-line no-new
            new URL(s);
            return true;
        }
        catch (_) { }
    }
    return false;
}
exports.isUrl = isUrl;
function styleSizeToNumber(s) {
    return s && s.endsWith('px') ? parseInt(s) : null;
}
exports.styleSizeToNumber = styleSizeToNumber;
function extractFilenameFromUrl(u, fallback, opts) {
    try {
        const { pathname } = new URL(u);
        const unsafeFilename = decodeURIComponent(pathname.split('/').pop() || fallback);
        const safeFilename = sanitize_filename_1.default(unsafeFilename);
        if ((opts === null || opts === void 0 ? void 0 : opts.preferredExt) && opts.preferredExt.length > 0) {
            const ext = (safeFilename.match(/\..+?$/) || [])[0];
            if (!ext || !opts.preferredExt.includes(ext)) {
                return safeFilename + opts.preferredExt[0];
            }
        }
        return safeFilename;
    }
    catch (_) { }
    return fallback;
}
exports.extractFilenameFromUrl = extractFilenameFromUrl;
//# sourceMappingURL=util.js.map